@@ -34,6 +34,10 @@
 #include <libsoup/soup.h>
 #include <string.h>
 
+#ifdef HAVE_GNOME
+#include <libsoup/soup-gnome-features.h>
+#endif /* HAVE_GNOME */
+
 #include "gdata-service.h"
 #include "gdata-private.h"
 #include "gdata-marshal.h"
@@ -60,7 +64,7 @@ static void gdata_service_get_property (GObject *object, guint property_id, GVal
 static void gdata_service_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static gboolean real_parse_authentication_response (GDataService *self, guint status, const gchar *response_body, gint length, GError **error);
 static void real_append_query_headers (GDataService *self, SoupMessage *message);
-static void real_parse_error_response (GDataService *self, guint status, const gchar *reason_phrase,
+static void real_parse_error_response (GDataService *self, GDataServiceError error_type, guint status, const gchar *reason_phrase,
 				       const gchar *response_body, gint length, GError **error);
 static void notify_proxy_uri_cb (GObject *gobject, GParamSpec *pspec, GObject *self);
 
@@ -106,6 +110,7 @@ gdata_service_class_init (GDataServiceClass *klass)
 
 	klass->service_name = "xapi";
 	klass->authentication_uri = "https://www.google.com/accounts/ClientLogin";
+	klass->feed_type = GDATA_TYPE_FEED;
 	klass->parse_authentication_response = real_parse_authentication_response;
 	klass->append_query_headers = real_append_query_headers;
 	klass->parse_error_response = real_parse_error_response;
@@ -195,6 +200,10 @@ gdata_service_init (GDataService *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_SERVICE, GDataServicePrivate);
 	self->priv->session = soup_session_sync_new ();
 
+#ifdef HAVE_GNOME
+	soup_session_add_feature_by_type (self->priv->session, SOUP_TYPE_GNOME_FEATURES_2_26);
+#endif /* HAVE_GNOME */
+
 	/* Proxy the SoupSession's proxy-uri property */
 	g_signal_connect (self->priv->session, "notify::proxy-uri", (GCallback) notify_proxy_uri_cb, self);
 }
@@ -319,10 +328,57 @@ real_append_query_headers (GDataService *self, SoupMessage *message)
 }
 
 static void
-real_parse_error_response (GDataService *self, guint status, const gchar *reason_phrase, const gchar *response_body, gint length, GError **error)
+real_parse_error_response (GDataService *self, GDataServiceError error_type, guint status, const gchar *reason_phrase, const gchar *response_body,
+			   gint length, GError **error)
 {
-	g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_QUERY,
-		     _("Error code %u when querying: %s"), status, reason_phrase);
+	/* See: http://code.google.com/apis/gdata/docs/2.0/reference.html#HTTPStatusCodes */
+
+	switch (status) {
+		case 400:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+				     _("Invalid request URI or header, or unsupported nonstandard parameter: %s"), reason_phrase);
+			return;
+		case 401:
+		case 403:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+				     _("Authentication required: %s"), reason_phrase);
+			return;
+		case 404:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_NOT_FOUND,
+				     _("The requested resource was not found: %s"), reason_phrase);
+			return;
+		case 409:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT,
+				     _("The entry has been modified since it was downloaded: %s"), reason_phrase);
+			return;
+		case 500:
+		default:
+			/* We'll fall back to generic errors, below */
+			break;
+	}
+
+	/* If the error hasn't been handled already, throw a generic error corresponding to the action type */
+	switch (error_type) {
+		case GDATA_SERVICE_ERROR_WITH_INSERTION:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_INSERTION,
+				     _("Error code %u when inserting an entry: %s"), status, reason_phrase);
+			break;
+		case GDATA_SERVICE_ERROR_WITH_UPDATE:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPDATE,
+				     _("Error code %u when updating an entry: %s"), status, reason_phrase);
+			break;
+		case GDATA_SERVICE_ERROR_WITH_DELETION:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DELETION,
+				     _("Error code %u when deleting an entry: %s"), status, reason_phrase);
+			break;
+		case GDATA_SERVICE_ERROR_WITH_QUERY:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_QUERY,
+				     _("Error code %u when querying: %s"), status, reason_phrase);
+			break;
+		default:
+			/* We should not be called with anything other than the above four generic error types */
+			g_assert_not_reached ();
+	}
 }
 
 typedef struct {
@@ -516,6 +572,10 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 		const gchar *response_body = message->response_body->data;
 		gchar *error_start, *error_end, *uri_start, *uri_end, *uri = NULL;
 
+		/* Parse the error response; see: http://code.google.com/apis/accounts/docs/AuthForInstalledApps.html#Errors */
+		if (response_body == NULL)
+			goto protocol_error;
+
 		/* Error */
 		error_start = strstr (response_body, "Error=");
 		if (error_start == NULL)
@@ -553,6 +613,7 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 			g_free (captcha_uri);
 
 			if (new_captcha_answer == NULL || *new_captcha_answer == '\0') {
+				/* Translators: see http://en.wikipedia.org/wiki/CAPTCHA for information about CAPTCHAs */
 				g_set_error_literal (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_CAPTCHA_REQUIRED,
 						     _("A CAPTCHA must be filled out to log in."));
 				goto login_error;
@@ -574,9 +635,9 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 			return authenticate (self, username, password, g_strndup (captcha_start, captcha_end - captcha_start), new_captcha_answer,
 					     cancellable, error);
 		} else if (strncmp (error_start, "Unknown", error_end - error_start) == 0) {
-			goto protocol_error; /* TODO: is this really a protocol error? It's an error with *our* code */
+			goto protocol_error;
 		} else if (strncmp (error_start, "BadAuthentication", error_end - error_start) == 0) {
-			/* TODO: Looks like Error=BadAuthentication errors don't return a URI */
+			/* Looks like Error=BadAuthentication errors don't return a URI */
 			g_set_error_literal (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_BAD_AUTHENTICATION,
 					     _("Your username or password were incorrect."));
 			goto login_error;
@@ -722,7 +783,7 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GError **
 		SoupURI *new_uri;
 		const gchar *new_location;
 
-		new_location = soup_message_headers_get (message->response_headers, "Location");
+		new_location = soup_message_headers_get_one (message->response_headers, "Location");
 		g_return_val_if_fail (new_location != NULL, SOUP_STATUS_NONE);
 
 		new_uri = soup_uri_new_with_base (soup_message_get_uri (message), new_location);
@@ -798,7 +859,7 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: data to pass to the @progress_callback function
- * @callback: a #GAsyncReadyCallback to call when authentication is finished
+ * @callback: a #GAsyncReadyCallback to call when the query is finished
  * @user_data: data to pass to the @callback function
  *
  * Queries the service's @feed_uri feed to build a #GDataFeed. @self, @feed_uri and
@@ -842,7 +903,7 @@ gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery
  *
  * Finishes an asynchronous query operation started with gdata_service_query_async().
  *
- * Return value: a #GDataFeed of query results; unref with g_object_unref()
+ * Return value: a #GDataFeed of query results, or %NULL; unref with g_object_unref()
  **/
 GDataFeed *
 gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GError **error)
@@ -893,7 +954,7 @@ gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GErr
  * If the #GDataQuery's ETag is set and it finds a match on the server, %FALSE will be returned, but @error will remain unset. Otherwise,
  * @query's ETag will be updated with the ETag from the returned feed, if available.
  *
- * Return value: a #GDataFeed of query results; unref with g_object_unref()
+ * Return value: a #GDataFeed of query results, or %NULL; unref with g_object_unref()
  **/
 GDataFeed *
 gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
@@ -907,6 +968,8 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 	GDataLink *link;
 
 	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (feed_uri != NULL, NULL);
+	g_return_val_if_fail (entry_type != G_TYPE_INVALID, NULL);
 
 	if (query != NULL) {
 		query_uri = gdata_query_get_query_uri (query, feed_uri);
@@ -941,14 +1004,15 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 	} else if (status != 200) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (self, status, message->reason_phrase, message->response_body->data, message->response_body->length, error);
+		klass->parse_error_response (self, GDATA_SERVICE_ERROR_WITH_QUERY, status, message->reason_phrase, message->response_body->data,
+					     message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
 
 	g_assert (message->response_body->data != NULL);
 
-	feed = _gdata_feed_new_from_xml (message->response_body->data, message->response_body->length, entry_type,
+	feed = _gdata_feed_new_from_xml (klass->feed_type, message->response_body->data, message->response_body->length, entry_type,
 					 progress_callback, progress_user_data, error);
 	g_object_unref (message);
 
@@ -969,6 +1033,121 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 	return feed;
 }
 
+typedef struct {
+	gchar *upload_uri;
+	GDataEntry *entry;
+} InsertEntryAsyncData;
+
+static void
+insert_entry_async_data_free (InsertEntryAsyncData *self)
+{
+	g_free (self->upload_uri);
+	if (self->entry)
+		g_object_unref (self->entry);
+
+	g_slice_free (InsertEntryAsyncData, self);
+}
+
+static void
+insert_entry_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *cancellable)
+{
+	GDataEntry *updated_entry;
+	GError *error = NULL;
+	InsertEntryAsyncData *data = g_simple_async_result_get_op_res_gpointer (result);
+
+	/* Check to see if it's been cancelled already */
+	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Insert the entry and return */
+	updated_entry = gdata_service_insert_entry (service, data->upload_uri, data->entry, cancellable, &error);
+	if (updated_entry == NULL) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Swap the old entry with the new one */
+	g_object_unref (data->entry);
+	data->entry = updated_entry;
+}
+
+/**
+ * gdata_service_insert_entry_async:
+ * @self: a #GDataService
+ * @upload_uri: the URI to which the upload should be sent
+ * @entry: the #GDataEntry to insert
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when insertion is finished
+ * @user_data: data to pass to the @callback function
+ *
+ * Inserts @entry by uploading it to the online service at @upload_uri. @self, @upload_uri and
+ * @entry are all reffed/copied when this function is called, so can safely be freed after this function returns.
+ *
+ * For more details, see gdata_service_insert_entry(), which is the synchronous version of this function.
+ *
+ * When the operation is finished, @callback will be called. You can then call gdata_service_insert_entry_finish()
+ * to get the results of the operation.
+ *
+ * Since: 0.3.0
+ **/
+void
+gdata_service_insert_entry_async (GDataService *self, const gchar *upload_uri, GDataEntry *entry, GCancellable *cancellable,
+				  GAsyncReadyCallback callback, gpointer user_data)
+{
+	GSimpleAsyncResult *result;
+	InsertEntryAsyncData *data;
+
+	g_return_if_fail (GDATA_IS_SERVICE (self));
+	g_return_if_fail (upload_uri != NULL);
+	g_return_if_fail (GDATA_IS_ENTRY (entry));
+
+	data = g_slice_new (InsertEntryAsyncData);
+	data->upload_uri = g_strdup (upload_uri);
+	data->entry = g_object_ref (entry);
+
+	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_insert_entry_async);
+	g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify) insert_entry_async_data_free);
+	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) insert_entry_thread, G_PRIORITY_DEFAULT, cancellable);
+	g_object_unref (result);
+}
+
+/**
+ * gdata_service_insert_entry_finish:
+ * @self: a #GDataService
+ * @async_result: a #GAsyncResult
+ * @error: a #GError, or %NULL
+ *
+ * Finishes an asynchronous entry insertion operation started with gdata_service_insert_entry_async().
+ *
+ * Return value: an updated #GDataEntry, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.3.0
+ **/
+GDataEntry *
+gdata_service_insert_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error)
+{
+	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
+	InsertEntryAsyncData *data;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), NULL);
+
+	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == gdata_service_insert_entry_async);
+
+	if (g_simple_async_result_propagate_error (result, error) == TRUE)
+		return NULL;
+
+	data = g_simple_async_result_get_op_res_gpointer (result);
+	if (data->entry != NULL)
+		return g_object_ref (data->entry);
+
+	g_assert_not_reached ();
+}
+
 /**
  * gdata_service_insert_entry:
  * @self: a #GDataService
@@ -1039,9 +1218,9 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 
 	if (status != 201) {
 		/* Error */
-		/* TODO: Handle errors more specifically, making sure to set authenticated where appropriate */
-		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_INSERTION,
-			     _("TODO: error code %u when inserting"), status);
+		g_assert (klass->parse_error_response != NULL);
+		klass->parse_error_response (self, GDATA_SERVICE_ERROR_WITH_INSERTION, status, message->reason_phrase, message->response_body->data,
+					     message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
@@ -1056,6 +1235,96 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 	return updated_entry;
 }
 
+static void
+update_entry_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *cancellable)
+{
+	GDataEntry *updated_entry;
+	GError *error = NULL;
+
+	/* Check to see if it's been cancelled already */
+	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Update the entry and return */
+	updated_entry = gdata_service_update_entry (service, g_simple_async_result_get_op_res_gpointer (result), cancellable, &error);
+	if (updated_entry == NULL) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Swap the old entry with the new one */
+	g_simple_async_result_set_op_res_gpointer (result, updated_entry, (GDestroyNotify) g_object_unref);
+}
+
+/**
+ * gdata_service_update_entry_async:
+ * @self: a #GDataService
+ * @entry: the #GDataEntry to update
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when the update is finished
+ * @user_data: data to pass to the @callback function
+ *
+ * Updates @entry by PUTting it to its <literal>edit</literal> link's URI. @self and
+ * @entry are both reffed when this function is called, so can safely be unreffed after this function returns.
+ *
+ * For more details, see gdata_service_update_entry(), which is the synchronous version of this function.
+ *
+ * When the operation is finished, @callback will be called. You can then call gdata_service_update_entry_finish()
+ * to get the results of the operation.
+ *
+ * Since: 0.3.0
+ **/
+void
+gdata_service_update_entry_async (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
+{
+	GSimpleAsyncResult *result;
+
+	g_return_if_fail (GDATA_IS_SERVICE (self));
+	g_return_if_fail (GDATA_IS_ENTRY (entry));
+
+	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_update_entry_async);
+	g_simple_async_result_set_op_res_gpointer (result, g_object_ref (entry), (GDestroyNotify) g_object_unref);
+	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) update_entry_thread, G_PRIORITY_DEFAULT, cancellable);
+	g_object_unref (result);
+}
+
+/**
+ * gdata_service_update_entry_finish:
+ * @self: a #GDataService
+ * @async_result: a #GAsyncResult
+ * @error: a #GError, or %NULL
+ *
+ * Finishes an asynchronous entry update operation started with gdata_service_update_entry_async().
+ *
+ * Return value: an updated #GDataEntry, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.3.0
+ **/
+GDataEntry *
+gdata_service_update_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error)
+{
+	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
+	GDataEntry *entry;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), NULL);
+
+	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == gdata_service_update_entry_async);
+
+	if (g_simple_async_result_propagate_error (result, error) == TRUE)
+		return NULL;
+
+	entry = g_simple_async_result_get_op_res_gpointer (result);
+	if (entry != NULL)
+		return g_object_ref (entry);
+
+	g_assert_not_reached ();
+}
+
 /**
  * gdata_service_update_entry:
  * @self: a #GDataService
@@ -1125,9 +1394,9 @@ gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable
 
 	if (status != 200) {
 		/* Error */
-		/* TODO: Handle errors more specifically, making sure to set authenticated where appropriate */
-		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPDATE,
-			     _("TODO: error code %u when updating"), status);
+		g_assert (klass->parse_error_response != NULL);
+		klass->parse_error_response (self, GDATA_SERVICE_ERROR_WITH_UPDATE, status, message->reason_phrase, message->response_body->data,
+					     message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
@@ -1142,6 +1411,91 @@ gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable
 	return updated_entry;
 }
 
+static void
+delete_entry_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *cancellable)
+{
+	gboolean success;
+	GError *error = NULL;
+
+	/* Check to see if it's been cancelled already */
+	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Delete the entry and return */
+	success = gdata_service_delete_entry (service, g_simple_async_result_get_op_res_gpointer (result), cancellable, &error);
+	if (success == FALSE) {
+		g_simple_async_result_set_from_error (result, error);
+		g_error_free (error);
+		return;
+	}
+
+	/* Replace the entry with the success value */
+	g_simple_async_result_set_op_res_gboolean (result, success);
+}
+
+/**
+ * gdata_service_delete_entry_async:
+ * @self: a #GDataService
+ * @entry: the #GDataEntry to delete
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @callback: a #GAsyncReadyCallback to call when deletion is finished
+ * @user_data: data to pass to the @callback function
+ *
+ * Deletes @entry from the server. @self and @entry are both reffed when this function is called,
+ * so can safely be unreffed after this function returns.
+ *
+ * For more details, see gdata_service_delete_entry(), which is the synchronous version of this function.
+ *
+ * When the operation is finished, @callback will be called. You can then call gdata_service_delete_entry_finish()
+ * to get the results of the operation.
+ *
+ * Since: 0.3.0
+ **/
+void
+gdata_service_delete_entry_async (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GAsyncReadyCallback callback, gpointer user_data)
+{
+	GSimpleAsyncResult *result;
+
+	g_return_if_fail (GDATA_IS_SERVICE (self));
+	g_return_if_fail (GDATA_IS_ENTRY (entry));
+
+	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_delete_entry_async);
+	g_simple_async_result_set_op_res_gpointer (result, g_object_ref (entry), (GDestroyNotify) g_object_unref);
+	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) delete_entry_thread, G_PRIORITY_DEFAULT, cancellable);
+	g_object_unref (result);
+}
+
+/**
+ * gdata_service_delete_entry_finish:
+ * @self: a #GDataService
+ * @async_result: a #GAsyncResult
+ * @error: a #GError, or %NULL
+ *
+ * Finishes an asynchronous entry deletion operation started with gdata_service_delete_entry_async().
+ *
+ * Return value: %TRUE on success, %FALSE otherwise
+ *
+ * Since: 0.3.0
+ **/
+gboolean
+gdata_service_delete_entry_finish (GDataService *self, GAsyncResult *async_result, GError **error)
+{
+	GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (async_result);
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), FALSE);
+	g_return_val_if_fail (G_IS_ASYNC_RESULT (async_result), FALSE);
+
+	g_warn_if_fail (g_simple_async_result_get_source_tag (result) == gdata_service_delete_entry_async);
+
+	if (g_simple_async_result_propagate_error (result, error) == TRUE)
+		return FALSE;
+
+	return g_simple_async_result_get_op_res_gboolean (result);
+}
+
 /**
  * gdata_service_delete_entry:
  * @self: a #GDataService
@@ -1190,22 +1544,28 @@ gdata_service_delete_entry (GDataService *self, GDataEntry *entry, GCancellable
 
 	/* Send the message */
 	status = _gdata_service_send_message (self, message, error);
-	g_object_unref (message);
-	if (status == SOUP_STATUS_NONE)
+	if (status == SOUP_STATUS_NONE) {
+		g_object_unref (message);
 		return FALSE;
+	}
 
 	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE)
+	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
+		g_object_unref (message);
 		return FALSE;
+	}
 
 	if (status != 200) {
 		/* Error */
-		/* TODO: Handle errors more specifically, making sure to set authenticated where appropriate */
-		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DELETION,
-			     _("TODO: error code %u when deleting"), status);
+		g_assert (klass->parse_error_response != NULL);
+		klass->parse_error_response (self, GDATA_SERVICE_ERROR_WITH_DELETION, status, message->reason_phrase, message->response_body->data,
+					     message->response_body->length, error);
+		g_object_unref (message);
 		return FALSE;
 	}
 
+	g_object_unref (message);
+
 	return TRUE;
 }
 
@@ -1268,14 +1628,14 @@ gdata_service_set_proxy_uri (GDataService *self, SoupURI *proxy_uri)
 gboolean
 gdata_service_is_authenticated (GDataService *self)
 {
-	g_assert (GDATA_IS_SERVICE (self));
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), FALSE);
 	return self->priv->authenticated;
 }
 
 void
 _gdata_service_set_authenticated (GDataService *self, gboolean authenticated)
 {
-	g_assert (GDATA_IS_SERVICE (self));
+	g_return_if_fail (GDATA_IS_SERVICE (self));
 	self->priv->authenticated = authenticated;
 }
 
