@@ -28,6 +28,9 @@
 #include <gdata/gdata-parser.h>
 #include <gdata/gdata-query.h>
 #include <gdata/gdata-enums.h>
+#include <gdata/gdata-access-handler.h>
+#include <gdata/gdata-access-rule.h>
+#include <gdata/gdata-parsable.h>
 
 /* Namespaces */
 #include <gdata/gdata-atom.h>
@@ -38,12 +41,14 @@
 
 /* YouTube */
 #include <gdata/services/youtube/gdata-youtube-service.h>
+#include <gdata/services/youtube/gdata-youtube-query.h>
 #include <gdata/services/youtube/gdata-youtube-video.h>
 #include <gdata/services/youtube/gdata-youtube.h>
 #include <gdata/services/youtube/gdata-youtube-enums.h>
 
 /* Google Calendar */
 #include <gdata/services/calendar/gdata-calendar-service.h>
+#include <gdata/services/calendar/gdata-calendar-feed.h>
 #include <gdata/services/calendar/gdata-calendar-calendar.h>
 #include <gdata/services/calendar/gdata-calendar-event.h>
 #include <gdata/services/calendar/gdata-calendar-query.h>
