@@ -54,7 +54,7 @@ static void gdata_youtube_service_get_property (GObject *object, guint property_
 static void gdata_youtube_service_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static gboolean parse_authentication_response (GDataService *self, guint status, const gchar *response_body, gint length, GError **error);
 static void append_query_headers (GDataService *self, SoupMessage *message);
-static void parse_error_response (GDataService *self, guint status, const gchar *reason_phrase,
+static void parse_error_response (GDataService *self, GDataServiceError error_type, guint status, const gchar *reason_phrase,
 				  const gchar *response_body, gint length, GError **error);
 
 struct _GDataYouTubeServicePrivate {
@@ -222,7 +222,8 @@ append_query_headers (GDataService *self, SoupMessage *message)
 }
 
 static void
-parse_error_response (GDataService *self, guint status, const gchar *reason_phrase, const gchar *response_body, gint length, GError **error)
+parse_error_response (GDataService *self, GDataServiceError error_type, guint status, const gchar *reason_phrase, const gchar *response_body,
+		      gint length, GError **error)
 {
 	xmlDoc *doc;
 	xmlNode *node;
@@ -234,7 +235,7 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 		length = strlen (response_body);
 
 	/* Parse the XML */
-	doc = xmlReadMemory (response_body, length, "error.xml", NULL, 0);
+	doc = xmlReadMemory (response_body, length, "/dev/null", NULL, 0);
 	if (doc == NULL)
 		goto parent;
 
@@ -253,10 +254,10 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 	}
 
 	/* Parse the actual errors */
-	node = node->xmlChildrenNode;
+	node = node->children;
 	while (node != NULL) {
 		xmlChar *domain = NULL, *code = NULL, *location = NULL;
-		xmlNode *child_node = node->xmlChildrenNode;
+		xmlNode *child_node = node->children;
 
 		if (node->type == XML_TEXT_NODE) {
 			/* Skip text nodes; they're all whitespace */
@@ -273,11 +274,11 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 			}
 
 			if (xmlStrcmp (child_node->name, (xmlChar*) "domain") == 0)
-				domain = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
+				domain = xmlNodeListGetString (doc, child_node->children, TRUE);
 			else if (xmlStrcmp (child_node->name, (xmlChar*) "code") == 0)
-				code = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
+				code = xmlNodeListGetString (doc, child_node->children, TRUE);
 			else if (xmlStrcmp (child_node->name, (xmlChar*) "location") == 0)
-				location = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
+				location = xmlNodeListGetString (doc, child_node->children, TRUE);
 			else if (xmlStrcmp (child_node->name, (xmlChar*) "internalReason") != 0) {
 				/* Unknown element (ignore internalReason) */
 				if (*error == NULL) {
@@ -316,7 +317,7 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 						     _("You have made too many API calls recently. Please wait a few minutes and try again."));
 				} else if (xmlStrcmp (code, (xmlChar*) "too_many_entries") == 0) {
 					g_set_error (error, GDATA_YOUTUBE_SERVICE_ERROR, GDATA_YOUTUBE_SERVICE_ERROR_ENTRY_QUOTA_EXCEEDED,
-						     _("You have exceeded your entry quota with the entry \"%s\"."
+						     _("You have exceeded your entry quota with the entry \"%s\". "
 						       "Please delete some entries and try again."), location);
 				} else {
 					/* Protocol error */
@@ -346,7 +347,7 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 
 parent:
 	/* Chain up to the parent class */
-	GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->parse_error_response (self, status, reason_phrase,
+	GDATA_SERVICE_CLASS (gdata_youtube_service_parent_class)->parse_error_response (self, error_type, status, reason_phrase,
 											response_body, length, error);
 	return;
 }
@@ -537,7 +538,7 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
 	if (related_link == NULL) {
 		/* Erroring out is probably the safest thing to do */
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-				     _("The video didn't have a related videos <link>."));
+				     _("The video did not have a related videos <link>."));
 		return NULL;
 	}
 
@@ -578,7 +579,7 @@ gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTu
 		/* Erroring out is probably the safest thing to do */
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
 						     GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
-						     _("The video didn't have a related videos <link>."));
+						     _("The video did not have a related videos <link>."));
 		return;
 	}
 
@@ -728,7 +729,7 @@ gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo
 
 	if (status != 201) {
 		/* Error */
-		parse_error_response (GDATA_SERVICE (self), status, message->reason_phrase,
+		parse_error_response (GDATA_SERVICE (self), GDATA_SERVICE_ERROR_WITH_INSERTION, status, message->reason_phrase,
 				      message->response_body->data, message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
