@@ -47,7 +47,8 @@ static void gdata_youtube_video_finalize (GObject *object);
 static void gdata_youtube_video_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_youtube_video_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void get_xml (GDataEntry *entry, GString *xml_string);
-static const gchar *get_namespaces (GDataEntry *entry);
+static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
+static void get_namespaces (GDataEntry *entry, GHashTable *namespaces);
 
 struct _GDataYouTubeVideoPrivate {
 	guint view_count;
@@ -77,6 +78,8 @@ struct _GDataYouTubeVideoPrivate {
 	/* Other properties */
 	gboolean is_draft;
 	GDataYouTubeState *state;
+	GDataGDFeedLink *comments_feed_link;
+	GTimeVal recorded;
 };
 
 enum {
@@ -98,7 +101,9 @@ enum {
 	PROP_UPLOADED,
 	PROP_VIDEO_ID,
 	PROP_IS_DRAFT,
-	PROP_STATE
+	PROP_STATE,
+	PROP_COMMENTS_FEED_LINK,
+	PROP_RECORDED
 };
 
 G_DEFINE_TYPE (GDataYouTubeVideo, gdata_youtube_video, GDATA_TYPE_ENTRY)
@@ -108,6 +113,7 @@ static void
 gdata_youtube_video_class_init (GDataYouTubeVideoClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GDataParsableClass *parsable_class = GDATA_PARSABLE_CLASS (klass);
 	GDataEntryClass *entry_class = GDATA_ENTRY_CLASS (klass);
 
 	g_type_class_add_private (klass, sizeof (GDataYouTubeVideoPrivate));
@@ -116,6 +122,8 @@ gdata_youtube_video_class_init (GDataYouTubeVideoClass *klass)
 	gobject_class->get_property = gdata_youtube_video_get_property;
 	gobject_class->finalize = gdata_youtube_video_finalize;
 
+	parsable_class->parse_xml = parse_xml;
+
 	entry_class->get_xml = get_xml;
 	entry_class->get_namespaces = get_namespaces;
 
@@ -344,7 +352,7 @@ gdata_youtube_video_class_init (GDataYouTubeVideoClass *klass)
 	g_object_class_install_property (gobject_class, PROP_UPLOADED,
 				g_param_spec_boxed ("uploaded",
 					"Uploaded", "Specifies the time the video was originally uploaded to YouTube.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -387,6 +395,37 @@ gdata_youtube_video_class_init (GDataYouTubeVideoClass *klass)
 				g_param_spec_pointer ("state",
 					"State", "Information describing the state of the video.",
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataYouTubeVideo:comments-feed-link:
+	 *
+	 * A link to the video's comments feed. It points to a #GDataGDFeedLink.
+	 *
+	 * For more information, see the <ulink type="http"
+	 * url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_gd:feedLink">online documentation</ulink>.
+	 *
+	 * Since: 0.3.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_COMMENTS_FEED_LINK,
+				g_param_spec_pointer ("comments-feed-link",
+					"Comments feed link", "A link to the video's comments feed.",
+					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataYouTubeVideo:recorded:
+	 *
+	 * Specifies the time the video was originally recorded.
+	 *
+	 * For more information, see the <ulink type="http"
+	 * url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_yt:recorded">online documentation</ulink>.
+	 *
+	 * Since: 0.3.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_RECORDED,
+				g_param_spec_boxed ("recorded",
+					"Recorded", "Specifies the time the video was originally recorded.",
+					GDATA_TYPE_G_TIME_VAL,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -418,6 +457,7 @@ gdata_youtube_video_finalize (GObject *object)
 
 	g_free (priv->video_id);
 	gdata_youtube_state_free (priv->state);
+	gdata_gd_feed_link_free (priv->comments_feed_link);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_youtube_video_parent_class)->finalize (object);
@@ -486,6 +526,12 @@ gdata_youtube_video_get_property (GObject *object, guint property_id, GValue *va
 		case PROP_STATE:
 			g_value_set_pointer (value, priv->state);
 			break;
+		case PROP_COMMENTS_FEED_LINK:
+			g_value_set_pointer (value, priv->comments_feed_link);
+			break;
+		case PROP_RECORDED:
+			g_value_set_boxed (value, &(priv->recorded));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -505,9 +551,6 @@ gdata_youtube_video_set_property (GObject *object, guint property_id, const GVal
 		case PROP_NO_EMBED:
 			gdata_youtube_video_set_no_embed (self, g_value_get_boolean (value));
 			break;
-		/*case PROP_RATING:
-			gdata_youtube_video_set_rating (self, g_value_get_pointer (value));
-			break;*/
 		case PROP_KEYWORDS:
 			gdata_youtube_video_set_keywords (self, g_value_get_string (value));
 			break;
@@ -526,6 +569,9 @@ gdata_youtube_video_set_property (GObject *object, guint property_id, const GVal
 		case PROP_IS_DRAFT:
 			gdata_youtube_video_set_is_draft (self, g_value_get_boolean (value));
 			break;
+		case PROP_RECORDED:
+			gdata_youtube_video_set_recorded (self, g_value_get_boxed (value));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -535,15 +581,16 @@ gdata_youtube_video_set_property (GObject *object, guint property_id, const GVal
 
 /**
  * gdata_youtube_video_new:
+ * @id: the video's ID, or %NULL
  *
- * Creates a new #GDataYouTubeVideo with default properties.
+ * Creates a new #GDataYouTubeVideo with the given ID and default properties.
  *
  * Return value: a new #GDataYouTubeVideo; unref with g_object_unref()
  **/
 GDataYouTubeVideo *
-gdata_youtube_video_new (void)
+gdata_youtube_video_new (const gchar *id)
 {
-	return g_object_new (GDATA_TYPE_YOUTUBE_VIDEO, NULL);
+	return g_object_new (GDATA_TYPE_YOUTUBE_VIDEO, "id", id, NULL);
 }
 
 /**
@@ -562,66 +609,7 @@ gdata_youtube_video_new (void)
 GDataYouTubeVideo *
 gdata_youtube_video_new_from_xml (const gchar *xml, gint length, GError **error)
 {
-	xmlDoc *doc;
-	xmlNode *node;
-
-	g_return_val_if_fail (xml != NULL, NULL);
-
-	if (length == -1)
-		length = strlen (xml);
-
-	/* Parse the XML */
-	doc = xmlReadMemory (xml, length, "entry.xml", NULL, 0);
-	if (doc == NULL) {
-		xmlError *xml_error = xmlGetLastError ();
-		g_set_error (error, GDATA_PARSER_ERROR, GDATA_PARSER_ERROR_PARSING_STRING,
-			     _("Error parsing XML: %s"),
-			     xml_error->message);
-		return NULL;
-	}
-
-	/* Get the root element */
-	node = xmlDocGetRootElement (doc);
-	if (node == NULL) {
-		/* XML document's empty */
-		xmlFreeDoc (doc);
-		g_set_error (error, GDATA_PARSER_ERROR, GDATA_PARSER_ERROR_EMPTY_DOCUMENT,
-			     _("Error parsing XML: %s"),
-			     _("Empty document."));
-		return NULL;
-	}
-
-	if (xmlStrcmp (node->name, (xmlChar*) "entry") != 0) {
-		/* No <entry> element (required) */
-		xmlFreeDoc (doc);
-		gdata_parser_error_required_element_missing ("entry", "root", error);
-		return NULL;
-	}
-
-	return _gdata_youtube_video_new_from_xml_node (doc, node, error);
-}
-
-GDataYouTubeVideo *
-_gdata_youtube_video_new_from_xml_node (xmlDoc *doc, xmlNode *node, GError **error)
-{
-	GDataYouTubeVideo *video;
-
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (node != NULL, FALSE);
-	g_return_val_if_fail (xmlStrcmp (node->name, (xmlChar*) "entry") == 0, FALSE);
-
-	video = gdata_youtube_video_new ();
-
-	node = node->xmlChildrenNode;
-	while (node != NULL) {
-		if (_gdata_youtube_video_parse_xml_node (video, doc, node, error) == FALSE) {
-			g_object_unref (video);
-			return NULL;
-		}
-		node = node->next;
-	}
-
-	return video;
+	return GDATA_YOUTUBE_VIDEO (_gdata_entry_new_from_xml (GDATA_TYPE_YOUTUBE_VIDEO, xml, length, error));
 }
 
 static gboolean
@@ -629,17 +617,17 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 {
 	if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
 		/* media:title */
-		xmlChar *title = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *title = xmlNodeListGetString (doc, node->children, TRUE);
 		gdata_youtube_video_set_title (self, (gchar*) title);
 		xmlFree (title);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "description") == 0) {
 		/* media:description */
-		xmlChar *description = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *description = xmlNodeListGetString (doc, node->children, TRUE);
 		gdata_youtube_video_set_description (self, (gchar*) description);
 		xmlFree (description);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "keywords") == 0) {
 		/* media:keywords */
-		xmlChar *keywords = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *keywords = xmlNodeListGetString (doc, node->children, TRUE);
 		gdata_youtube_video_set_keywords (self, (gchar*) keywords);
 		xmlFree (keywords);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
@@ -649,9 +637,9 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 
 		scheme = xmlGetProp (node, (xmlChar*) "scheme");
 		label = xmlGetProp (node, (xmlChar*) "label");
-		content = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		content = xmlNodeListGetString (doc, node->children, TRUE);
 
-		category = gdata_media_category_new ((gchar*) content, (gchar*) scheme, (gchar*) label);
+		category = gdata_media_category_new ((gchar*) content, (gchar*) label, (gchar*) scheme);
 		gdata_youtube_video_set_category (self, category);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "content") == 0) {
 		/* media:content */
@@ -730,7 +718,7 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 			return FALSE;
 		}
 
-		content = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		content = xmlNodeListGetString (doc, node->children, TRUE);
 
 		gdata_media_credit_free (self->priv->credit);
 		self->priv->credit = gdata_media_credit_new ((gchar*) content, (type != NULL) ? TRUE : FALSE);
@@ -774,7 +762,7 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 		}
 		xmlFree (type);
 
-		countries = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		countries = xmlNodeListGetString (doc, node->children, TRUE);
 		relationship = xmlGetProp (node, (xmlChar*) "relationship");
 
 		if (xmlStrcmp (relationship, (xmlChar*) "allow") == 0)
@@ -793,7 +781,7 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 		g_object_notify (G_OBJECT (self), "restriction");
 	} else if (xmlStrcmp (node->name, (xmlChar*) "thumbnail") == 0) {
 		/* media:thumbnail */
-		xmlChar *uri, *width, *height, *time;
+		xmlChar *uri, *width, *height, *_time;
 		guint width_uint, height_uint;
 		gint time_int;
 		GDataMediaThumbnail *thumbnail;
@@ -812,19 +800,19 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 		xmlFree (height);
 
 		/* Get and parse the time */
-		time = xmlGetProp (node, (xmlChar*) "time");
-		if (time == NULL) {
+		_time = xmlGetProp (node, (xmlChar*) "time");
+		if (_time == NULL) {
 			time_int = -1;
 		} else {
-			time_int = gdata_media_thumbnail_parse_time ((gchar*) time);
+			time_int = gdata_media_thumbnail_parse_time ((gchar*) _time);
 			if (time_int == -1) {
 				g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
 					     _("The @time property (\"%s\") of a <media:thumbnail> could not be parsed."),
-					     (gchar*) time);
-				xmlFree (time);
+					     (gchar*) _time);
+				xmlFree (_time);
 				return FALSE;
 			}
-			xmlFree (time);
+			xmlFree (_time);
 		}
 
 		uri = xmlGetProp (node, (xmlChar*) "url");
@@ -848,7 +836,7 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 		xmlChar *uploaded;
 		GTimeVal uploaded_timeval;
 
-		uploaded = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		uploaded = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) uploaded, &uploaded_timeval) == FALSE) {
 			/* Error */
 			gdata_parser_error_not_iso8601_format ("uploaded", "media:group", (gchar*) uploaded, error);
@@ -861,7 +849,7 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 		g_object_notify (G_OBJECT (self), "uploaded");
 	} else if (xmlStrcmp (node->name, (xmlChar*) "videoid") == 0) {
 		/* yt:videoid */
-		xmlChar *video_id = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *video_id = xmlNodeListGetString (doc, node->children, TRUE);
 		g_free (self->priv->video_id);
 		self->priv->video_id = g_strdup ((gchar*) video_id);
 		g_object_notify (G_OBJECT (self), "video-id");
@@ -873,20 +861,22 @@ parse_media_group_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node,
 	return TRUE;
 }
 
-gboolean
-_gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNode *node, GError **error)
+static gboolean
+parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GError *child_error = NULL;
+	GDataYouTubeVideo *self;
 
-	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (self), FALSE);
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (parsable), FALSE);
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (node != NULL, FALSE);
 
+	self = GDATA_YOUTUBE_VIDEO (parsable);
+
 	if (xmlStrcmp (node->name, (xmlChar*) "group") == 0) {
 		/* media:group */
 		xmlNode *child_node;
 
-		child_node = node->xmlChildrenNode;
+		child_node = node->children;
 		while (child_node != NULL) {
 			if (parse_media_group_xml_node (self, doc, child_node, error) == FALSE)
 				return FALSE;
@@ -897,7 +887,6 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 		xmlChar *min, *max, *num_raters, *average;
 		guint num_raters_uint;
 		gdouble average_double;
-		/*GDataGDRating *rating;*/
 
 		min = xmlGetProp (node, (xmlChar*) "min");
 		if (min == NULL)
@@ -929,16 +918,14 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 							  strtoul ((gchar*) max, NULL, 10),
 							  num_raters_uint, average_double);
 		g_object_notify (G_OBJECT (self), "rating");
-		/*gdata_youtube_video_set_rating (self, rating);*/
 	} else if (xmlStrcmp (node->name, (xmlChar*) "comments") == 0) {
 		/* gd:comments */
-		xmlChar *rel, *href, *count_hint;
+		xmlChar *rel, *href, *count_hint, *read_only;
 		xmlNode *child_node;
 		guint count_hint_uint;
-		/*GDataGDFeedLink *feed_link;*/
 
 		/* This is actually the child of the <comments> element */
-		child_node = node->xmlChildrenNode;
+		child_node = node->children;
 
 		count_hint = xmlGetProp (child_node, (xmlChar*) "countHint");
 		if (count_hint == NULL)
@@ -947,15 +934,18 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 			count_hint_uint = strtoul ((gchar*) count_hint, NULL, 10);
 		xmlFree (count_hint);
 
+		read_only = xmlGetProp (child_node, (xmlChar*) "readOnly");
 		rel = xmlGetProp (child_node, (xmlChar*) "rel");
 		href = xmlGetProp (child_node, (xmlChar*) "href");
 
-		/* TODO */
-		/*feed_link = gdata_gd_feed_link_new ((gchar*) href, (gchar*) rel, count_hint_uint);*/
-		/*gdata_youtube_video_set_comments_feed_link (self, feed_link);*/
+		gdata_gd_feed_link_free (self->priv->comments_feed_link);
+		self->priv->comments_feed_link = gdata_gd_feed_link_new ((gchar*) href, (gchar*) rel, count_hint_uint,
+									 ((xmlStrcmp (read_only, (xmlChar*) "true") == 0) ? TRUE : FALSE));
+		g_object_notify (G_OBJECT (self), "comments-feed-link");
 
 		xmlFree (rel);
 		xmlFree (href);
+		xmlFree (read_only);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "statistics") == 0) {
 		/* yt:statistics */
 		xmlChar *view_count, *favorite_count;
@@ -978,7 +968,7 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 		xmlFree (favorite_count);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "location") == 0) {
 		/* yt:location */
-		xmlChar *location = xmlNodeListGetString (doc, node->xmlChildrenNode, TRUE);
+		xmlChar *location = xmlNodeListGetString (doc, node->children, TRUE);
 		gdata_youtube_video_set_location (self, (gchar*) location);
 		xmlFree (location);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "where") == 0) {
@@ -989,12 +979,23 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 		gdata_youtube_video_set_no_embed (self, TRUE);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "recorded") == 0) {
 		/* yt:recorded */
-		g_message ("TODO: recorded unimplemented");
+		xmlChar *recorded;
+		GTimeVal recorded_timeval;
+
+		recorded = xmlNodeListGetString (doc, node->children, TRUE);
+		if (gdata_parser_time_val_from_date ((gchar*) recorded, &recorded_timeval) == FALSE) {
+			/* Error */
+			gdata_parser_error_not_iso8601_format ("recorded", "entry", (gchar*) recorded, error);
+			xmlFree (recorded);
+			return FALSE;
+		}
+		xmlFree (recorded);
+		gdata_youtube_video_set_recorded (self, &recorded_timeval);
 	} else if (xmlStrcmp (node->name, (xmlChar*) "control") == 0) {
 		/* app:control */
 		xmlNode *child_node;
 
-		child_node = node->xmlChildrenNode;
+		child_node = node->children;
 		while (child_node != NULL) {
 			if (xmlStrcmp (child_node->name, (xmlChar*) "draft") == 0) {
 				/* app:draft */
@@ -1007,7 +1008,7 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 				if (name == NULL)
 					return gdata_parser_error_required_property_missing ("yt:state", "name", error);
 
-				message = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
+				message = xmlNodeListGetString (doc, child_node->children, TRUE);
 				reason_code = xmlGetProp (child_node, (xmlChar*) "reasonCode");
 				help_uri = xmlGetProp (child_node, (xmlChar*) "helpUrl");
 
@@ -1027,14 +1028,8 @@ _gdata_youtube_video_parse_xml_node (GDataYouTubeVideo *self, xmlDoc *doc, xmlNo
 
 			child_node = child_node->next;
 		}
-	} else if (_gdata_entry_parse_xml_node (GDATA_ENTRY (self), doc, node, &child_error) == FALSE) {
-		if (g_error_matches (child_error, GDATA_PARSER_ERROR, GDATA_PARSER_ERROR_UNHANDLED_XML_ELEMENT) == TRUE) {
-			g_error_free (child_error);
-			gdata_parser_error_unhandled_element ((gchar*) node->ns->prefix, (gchar*) node->name, "entry", error);
-		} else {
-			g_propagate_error (error, child_error);
-		}
-
+	} else if (GDATA_PARSABLE_CLASS (gdata_youtube_video_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
+		/* Error! */
 		return FALSE;
 	}
 
@@ -1046,9 +1041,7 @@ get_xml (GDataEntry *entry, GString *xml_string)
 {
 	GDataYouTubeVideoPrivate *priv = GDATA_YOUTUBE_VIDEO (entry)->priv;
 	gchar *category;
-	GList *contents, *thumbnails;
 
-	/* TODO: Make sure read-only properties aren't represented here */
 	/* Chain up to the parent class */
 	GDATA_ENTRY_CLASS (gdata_youtube_video_parent_class)->get_xml (entry, xml_string);
 
@@ -1082,61 +1075,6 @@ get_xml (GDataEntry *entry, GString *xml_string)
 		g_free (keywords);
 	}
 
-	for (contents = priv->contents; contents != NULL; contents = contents->next) {
-		GDataMediaContent *content = (GDataMediaContent*) contents->data;
-		gchar *uri, *type;
-
-		uri = g_markup_escape_text (content->uri, -1);
-		type = g_markup_escape_text (content->type, -1);
-		g_string_append_printf (xml_string, "<media:content url='%s' type='%s'", uri, type);
-		g_free (uri);
-		g_free (type);
-
-		if (content->is_default == TRUE)
-			g_string_append (xml_string, " isDefault='true'");
-		else
-			g_string_append (xml_string, " isDefault='false'");
-
-		switch (content->expression) {
-		case GDATA_MEDIA_EXPRESSION_SAMPLE:
-			g_string_append (xml_string, " expression='sample'");
-			break;
-		case GDATA_MEDIA_EXPRESSION_FULL:
-			g_string_append (xml_string, " expression='full'");
-			break;
-		case GDATA_MEDIA_EXPRESSION_NONSTOP:
-			g_string_append (xml_string, " expression='nonstop'");
-			break;
-		default:
-			g_assert_not_reached ();
-		}
-
-		if (content->duration != -1)
-			g_string_append_printf (xml_string, " duration='%i'", content->duration);
-		if (content->format != -1)
-			g_string_append_printf (xml_string, " yt:format='%i'", content->format);
-
-		g_string_append (xml_string, "/>");
-	}
-
-	if (priv->player_uri != NULL) {
-		gchar *player_uri = g_markup_escape_text (priv->player_uri, -1);
-		g_string_append_printf (xml_string, "<media:player url='%s'/>", player_uri);
-		g_free (player_uri);
-	}
-
-	for (thumbnails = priv->thumbnails; thumbnails != NULL; thumbnails = thumbnails->next) {
-		GDataMediaThumbnail *thumbnail = (GDataMediaThumbnail*) thumbnails->data;
-		gchar *uri, *time;
-
-		uri = g_markup_escape_text (thumbnail->uri, -1);
-		time = gdata_media_thumbnail_build_time (thumbnail->time);
-		g_string_append_printf (xml_string, "<media:thumbnail url='%s' height='%u' width='%u' time='%s'/>",
-					uri, thumbnail->height, thumbnail->width, time);
-		g_free (uri);
-		g_free (time);
-	}
-
 	if (priv->is_private == TRUE)
 		g_string_append (xml_string, "<yt:private/>");
 
@@ -1151,20 +1089,26 @@ get_xml (GDataEntry *entry, GString *xml_string)
 		g_free (location);
 	}
 
+	if (priv->recorded.tv_sec != 0 || priv->recorded.tv_usec != 0) {
+		gchar *recorded = gdata_parser_date_from_time_val (&(priv->recorded));
+		g_string_append_printf (xml_string, "<yt:recorded>%s</yt:recorded>", recorded);
+		g_free (recorded);
+	}
+
 	/* TODO:
-	 * - yt:recorded
 	 * - georss:where
-	 * - Check all tags here are valid for insertions and updates
 	 * - Check things are escaped (or not) as appropriate
-	 * - Write a function to encapsulate g_markup_escape_text and
-	 *   g_string_append_printf to reduce the number of allocations
 	 */
 }
 
-static const gchar *
-get_namespaces (GDataEntry *entry)
+static void
+get_namespaces (GDataEntry *entry, GHashTable *namespaces)
 {
-	return "xmlns:media='http://search.yahoo.com/mrss/' xmlns:yt='http://gdata.youtube.com/schemas/2007'";
+	/* Chain up to the parent class */
+	GDATA_ENTRY_CLASS (gdata_youtube_video_parent_class)->get_namespaces (entry, namespaces);
+
+	g_hash_table_insert (namespaces, (gchar*) "media", (gchar*) "http://search.yahoo.com/mrss/");
+	g_hash_table_insert (namespaces, (gchar*) "yt", (gchar*) "http://gdata.youtube.com/schemas/2007");
 }
 
 /**
@@ -1276,17 +1220,6 @@ gdata_youtube_video_get_rating (GDataYouTubeVideo *self)
 	return self->priv->rating;
 }
 
-/* TODO: The only use for this is for adding a new rating, but there must be a better interface for that */
-/*void
-gdata_youtube_video_set_rating (GDataYouTubeVideo *self, GDataGDRating *rating)
-{
-	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (self));
-
-	gdata_gd_rating_free (self->priv->rating);  TODO: Not so happy about this memory management 
-	self->priv->rating = rating;
-	g_object_notify (G_OBJECT (self), "rating");
-}*/
-
 /**
  * gdata_youtube_video_get_keywords:
  * @self: a #GDataYouTubeVideo
@@ -1422,7 +1355,8 @@ gdata_youtube_video_get_category (GDataYouTubeVideo *self)
  * @self: a #GDataYouTubeVideo
  * @category: a new #GDataMediaCategory
  *
- * Sets the #GDataYouTubeVideo:category property to the new category, @category.
+ * Sets the #GDataYouTubeVideo:category property to the new category, @category. The #GDataYouTubeVideo
+ * will take ownership of @category, so do not free it after returning from this function.
  *
  * @category must not be %NULL. For more information, see the <ulink type="http"
  * url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_media:category">online documentation</ulink>.
@@ -1433,7 +1367,7 @@ gdata_youtube_video_set_category (GDataYouTubeVideo *self, GDataMediaCategory *c
 	g_return_if_fail (category != NULL);
 	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (self));
 
-	gdata_media_category_free (self->priv->category); /* TODO: not so happy about this memory management */
+	gdata_media_category_free (self->priv->category);
 	self->priv->category = category;
 	g_object_notify (G_OBJECT (self), "category");
 }
@@ -1584,7 +1518,7 @@ gdata_youtube_video_set_is_private (GDataYouTubeVideo *self, gboolean is_private
  * @uploaded: a #GTimeVal
  *
  * Gets the #GDataYouTubeVideo:uploaded property and puts it in @uploaded. If the property is unset,
- * both fields in the #GTimeVal will be set to 0.
+ * both fields in the #GTimeVal will be set to %0.
  **/
 void
 gdata_youtube_video_get_uploaded (GDataYouTubeVideo *self, GTimeVal *uploaded)
@@ -1655,3 +1589,63 @@ gdata_youtube_video_get_state (GDataYouTubeVideo *self)
 	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (self), NULL);
 	return self->priv->state;
 }
+
+/**
+ * gdata_youtube_video_get_comments_feed_link:
+ * @self: a #GDataYouTubeVideo
+ *
+ * Gets the #GDataYouTubeVideo:comments-feed-link property.
+ *
+ * For more information, see the <ulink type="http"
+ * url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_gd:feedLink">online documentation</ulink>.
+ *
+ * Return value: a #GDataGDFeedLink to the video's comments feed, or %NULL
+ *
+ * Since: 0.3.0
+ **/
+GDataGDFeedLink *
+gdata_youtube_video_get_comments_feed_link (GDataYouTubeVideo *self)
+{
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_VIDEO (self), NULL);
+	return self->priv->comments_feed_link;
+}
+
+/**
+ * gdata_youtube_video_get_recorded:
+ * @self: a #GDataYouTubeVideo
+ * @recorded: a #GTimeVal
+ *
+ * Gets the #GDataYouTubeVideo:recorded property and puts it in @recorded. If the property is unset,
+ * both fields in the #GTimeVal will be set to %0.
+ *
+ * Since: 0.3.0
+ **/
+void
+gdata_youtube_video_get_recorded (GDataYouTubeVideo *self, GTimeVal *recorded)
+{
+	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (self));
+	*recorded = self->priv->recorded;
+}
+
+/**
+ * gdata_youtube_video_set_recorded:
+ * @self: a #GDataYouTubeVideo
+ * @recorded: the video's new recorded time
+ *
+ * Sets the #GDataYouTubeVideo:recorded property to the new recorded time, @recorded.
+ *
+ * Set @recorded to %NULL to unset the video's recorded time.
+ *
+ * Since: 0.3.0
+ **/
+void
+gdata_youtube_video_set_recorded (GDataYouTubeVideo *self, GTimeVal *recorded)
+{
+	g_return_if_fail (GDATA_IS_YOUTUBE_VIDEO (self));
+	if (recorded == NULL) {
+		self->priv->recorded.tv_sec = 0;
+		self->priv->recorded.tv_usec = 0;
+	} else {
+		self->priv->recorded = *recorded;
+	}
+}
